# -*- coding: utf-8 -*-
import csv
import os.path
from django.db import connection
from django.conf import settings
from django.shortcuts import render
from django.contrib.auth.decorators import login_required
from reportserver.core.views.export.dates2export import stringify_pair


@login_required
def overview(request):
    """
    View of the index page of authenticated users
    :param request:
    :return: index template for authenticated users
    """
    populate_session(request)
    return render(request, "index_auth.html", {"url_name": ""})


def populate_session(request):
    """
    Prepopulates session with nuts1, regioncategory and beneficiary from DB
    :param request:
    :return:
    """
    cursor = connection.cursor()
    cursor.execute("SELECT id, label FROM nuts1 ORDER BY id;")
    request.session["nuts1"] = cursor.fetchall()
    cursor.execute("SELECT id, label FROM regioncategory ORDER BY id;")
    request.session["regioncategory"] = cursor.fetchall()
    cursor.execute("SELECT id, label FROM beneficiary "
                   "WHERE fp IS NOT NULL "
                   "ORDER BY instance ASC, fp ASC")
    request.session["beneficiary"] = cursor.fetchall()
    cursor.execute("SELECT export_date FROM version")
    request.session["export_date"] = cursor.fetchone()[0].strftime("%d.%m.%Y")

    request.session["dates2_lut"] = get_datesII_lut_dict()


def cut_instanceid(value, separator=".", merger="."):
    split_value = value.split(separator)
    return merger.join(split_value[0:2])


def get_regioncategory_id_from(str1):
    regioncategory_dict = {"*": "*",
                           "1": "1",
                           "seR": "1",
                           "2": "2",
                           u"ÜR": "2",
                           "3": "3",
                           "Region Leipzig": "3",
                           "4": "4",
                           u"Region Lüneburg": "4"}
    return regioncategory_dict[str1]


def generate_lut_mapping(mappingfile):
    result_dict = dict()
    reader = csv.reader(mappingfile, delimiter=";")
    doubles = []
    for row in reader:
        u_row = [unicode(s, "utf-8") for s in row]
        instance_id = u_row[0].strip()
        regioncategory_id = get_regioncategory_id_from(u_row[1].strip())
        if stringify_pair(instance_id, regioncategory_id) \
                not in result_dict:
            result_dict[stringify_pair(instance_id, regioncategory_id)] = \
                u_row[2]
        else:
            doubles.append(stringify_pair(instance_id, regioncategory_id))
    if doubles:
        raise ValueError("In the dates2 lut table are following doubles: "
                         + repr(doubles))
    return result_dict


def get_datesII_lut_dict():
    path = settings.DATES2_LUT_PATH
    pathname = "DATES2_LUT_PATH"

    if not path:
        raise ValueError("%s Configuration missing" % pathname)
    if not os.path.isfile(path):
        raise ValueError("File '" + path + "' doesn't exist")

    with open(path, "rb") as mappingfile:
        return generate_lut_mapping(mappingfile)


def entry(request):
    """
    View of the index page of authenticated users
    :param request:
    :return: index template
    """
    if not settings.PROGRAM:
        raise ValueError("Missing configuration value: PROGRAM")
    return render(request, "index.html")
