from reportserver.lib.db import sqlfactory
from reportserver.lib.report.esfreport import EsfReport
from django.db import connection


def generate_sql(indicators, filters, group_by=None, additional_selects=None):
    """
    Generates query depending on indicators, start and end
    :param indicators: indicators which are summed over
    :param filters: filters used in the sql statement
    :return:
    """
    factory = sqlfactory.SQLFactory("eval_virtual_participations",
                                    start_date=filters.start,
                                    end_date=filters.end)
    factory.indicators = indicators
    basic_filters = []
    if filters.filter_type == "1":
        basic_filters.append(("start_date >= %s", filters.start))
        basic_filters.append(("end_date IS NOT NULL AND end_date <= %s",
                              filters.end))
    elif filters.filter_type == "2":
        basic_filters.append(("start_date >= %s", filters.start))
        basic_filters.append(("start_date <= %s", filters.end))
    elif filters.filter_type == "3":
        basic_filters.append(("end_date >= %s AND end_date IS NOT NULL",
                              filters.start))
        basic_filters.append(("end_date <= %s", filters.end))
    elif filters.filter_type == "4":
        basic_filters.append(("((start_date >= %s", filters.start))
        basic_filters.append(("start_date <= %s) OR (end_date IS NOT NULL ",
                              filters.end))
        basic_filters.append((" end_date >= %s ", filters.start))
        basic_filters.append((" end_date <= %s))", filters.end))
    where = basic_filters
    if filters.nuts1:
        where.append(("nuts1 = ANY(%s)", filters.nuts1))
    if filters.regioncategory:
        regioncategories = [int(v) for v in filters.regioncategory]
        where.append(("regioncategory_id = ANY(%s)", regioncategories))
    if filters.beneficiary:
        where.append(("beneficiary_id = ANY(%s)", filters.beneficiary))
    if filters.gender:
        gender = [int(v) for v in filters.gender]
        where.append(("gender = ANY(%s)", gender))

    if filters.foerderphase:
        foerderphase = [int(v) for v in filters.foerderphase]
        if 0 in foerderphase and 1 in foerderphase:
            pass
        elif 0 in foerderphase:
            where.append(["(start_date < '2019-01-01'::date)", []])
        elif 1 in foerderphase:
            where.append(["(start_date > '2018-12-31'::date)", []])

    factory.filters = where
    if group_by:
        factory.group_by = group_by
    if additional_selects:
        factory.additional_selects = additional_selects
    return factory.create()


def fetch_as_dictionaries(cursor):
    """
    :param cursor: cursor from the database
    :return: a list of dictionaries containing the colum-name as key and
             column-value as value
             [{column:value}, {column:value}]
    """
    columns = [col[0] for col in cursor.description]
    return [dict(zip(columns, row)) for row in cursor.fetchall()]


def retrieve_reportdata_for_export(filters, programname):
    indicators = EsfReport.indicators_for(programname)
    additional_selects = ["MAX(beneficiary_id) AS Vorhaben"]
    group_by = ["group by gender, beneficiary_id"]
    sql = generate_sql(indicators=indicators, filters=filters,
                       additional_selects=additional_selects,
                       group_by=group_by)
    cursor = connection.cursor()
    cursor.execute(*sql)
    rows = fetch_as_dictionaries(cursor)
    return rows


def retrieve_report_data(filters, programname):
    indicators = EsfReport.indicators_for(programname)
    sql = generate_sql(indicators=indicators, filters=filters)
    cursor = connection.cursor()
    cursor.execute(*sql)
    rows = fetch_as_dictionaries(cursor)
    return rows
